﻿//////////////////////////////////////////////
// TestUtils.h
// 
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkDebug
#include "../Dll/DllDefines.h"

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <algorithm>

/// Class ------------------------------------

namespace nkDebug
{
	class DLL_NKDEBUG_EXPORT TestUtils final
	{
		public :

			// Testing utilities
			static void check (bool condition, const nkMemory::StringView& message = "") ;

		public :

			// Template utilities
			template <typename T>
			static void areEqual (const T& a, const T& b, const nkMemory::StringView& message = "")
			{
				check(a == b, message) ;
			}

			template <typename T>
			static void areEqual (T a, T b, T precision, const nkMemory::StringView& message = "")
			{
				check(std::abs(a - b) <= precision, message) ;
			}

			template <typename T>
			static void areNotEqual (const T& a, const T& b, const nkMemory::StringView& message = "")
			{
				check(a != b, message) ;
			}

			template <typename T>
			static void areNotEqual (T a, T b, T precision, const nkMemory::StringView& message = "")
			{
				check(std::abs(a - b) > precision, message) ;
			}

			template <typename T>
			static void isNull (T* p, const nkMemory::StringView& message = "")
			{
				check(p, message) ;
			}

			template <typename T>
			static void isNotNull (T* p, const nkMemory::StringView& message ="")
			{
				check(!p, message) ;
			}

		public :

			// Specialized templates
			template <typename T = const char*>
			static void areEqual (const char* const& a, const char* const& b, const nkMemory::StringView& message = "")
			{
				check(nkMemory::StringView(a) == nkMemory::StringView(b), message) ;
			}

			template <typename T = const char*>
			static void areNotEqual (const char* const& a, const char* const& b, const nkMemory::StringView& message = "")
			{
				check(nkMemory::StringView(a) != nkMemory::StringView(b), message) ;
			}
	} ;
}